# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_sensor_data": "01_json_to_df.ipynb",
         "addAATData": "01_json_to_df.ipynb",
         "addSessionData": "01_json_to_df.ipynb",
         "makeParticipantTable": "01_json_to_df.ipynb",
         "relabel_variables": "01_json_to_df.ipynb",
         "json_to_df": "01_json_to_df.ipynb",
         "Constants": "02_constants.ipynb",
         "interpolate": "03_add_responses.ipynb",
         "butterFilter": "03_add_responses.ipynb",
         "find_first_peak": "03_add_responses.ipynb",
         "find_response": "03_add_responses.ipynb",
         "getResponses": "03_add_responses.ipynb",
         "add_responses": "03_add_responses.ipynb",
         "merge_data": "04_utils.ipynb",
         "loadJson": "04_utils.ipynb",
         "sorted_nicely": "05_make_condition_templates.ipynb",
         "getProperty": "05_make_condition_templates.ipynb",
         "makeConditionTable": "05_make_condition_templates.ipynb",
         "make_condition_templates_from_files": "05_make_condition_templates.ipynb",
         "make_condition_templates": "05_make_condition_templates.ipynb",
         "BASE_COLUMNS": "05_make_condition_templates.ipynb",
         "AAT_COLUMNS": "05_make_condition_templates.ipynb"}

modules = ["json_to_df.py",
           "constants.py",
           "add_responses.py",
           "utils.py",
           "make_condition_templates.py"]

doc_url = "https://hgzech.github.io/aat_analysis/"

git_url = "https://github.com/hgzech/aat_analysis/tree/master/"

def custom_doc_links(name): return None
