# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/02_constants.ipynb (unless otherwise specified).

__all__ = ['Constants']

# Cell
class Constants():
        ACCELERATION_COLUMN = 'accelerations'
        ACCELERATION_X_COLUMN = 'accelerations_x'
        ACCELERATION_Y_COLUMN = 'accelerations_y'
        ACCURACY_COLUMN = 'accuracy'
        BLOCK_COLUMN = "block"
        BUTTERED_COLUMN = 'buttered'
        CONDITION_COLUMN = "condition"
        CORRECT_RESPONSE_COLUMN = 'correct_response'
        DEVICE_COLUMN = 'device'
        DRAWN_AT_COLUMN = 'drawn_at'
        DRAWN_AT_UNIX_COLUMN = 'drawn_at_unix'
        EXPERIMENT_COLUMN = 'experiment'
        FORCE_COLUMN = 'force'
        GYRO_TIME_COLUMN = 'gyro_times'
        GYRO_X_COLUMN = 'gyro_x'
        GYRO_Y_COLUMN = 'gyro_y'
        GYRO_Z_COLUMN = 'gyro_z'
        INTERPOLATED_COLUMN = 'interpolated'
        INTERPOLATED_GYRO_COLUMN = 'interpolated_gyro'
        IS_PRACTICE_COLUMN = 'is_practice'
        NUMERICAL_QUESTION_TYPES = ['likert']
        CATEGORICAL_QUESTION_TYPES = ['multiple','text_input']
        PARTICIPANT_COLUMN = 'participant'
        PEAK_AT_COLUMN = 'peak_at'
        RESPONSE_COLUMN = 'response'
        RT_COLUMN = 'rt'
        RT_INV_COLUMN = 'rt_inv'
        SENSOR_TYPE_COLUMN = 'sensor_type'
        SESSION_COLUMN = "session"
        SESSION_NUMBER_COLUMN = "session_number"
        SIGNED_UP_COLUMN = 'signed_up'
        STIMULUS_COLUMN = 'stimulus'
        STIMULUS_SET_COLUMN = 'stimulus_set'
        TIME_COLUMN = 'times'
        TIME_CORRECTION_COLUMN = 'time_correction'
        TRIAL_NUMBER_COLUMN = 'block_trial'
        TRIAL_NUMBER_CUM_COLUMN = 'trial'
        V_FINAL_COLUMN = 'v_final'
        V_MAX_COLUMN = 'v_max'